import re
import subprocess
import DefaultModule

class LicenseCheck(DefaultModule.DefaultModule):
    """
    Uses rpmdev-extract tool to extract rpm's content and license-check
    utility to check licenses usage. It parses licensecheck's output
    and does some wild heuristics about licensing usage.
    """

    _errors_to_ignore = [ ]
    _errors_to_ignore_scl = [ "dir-or-file-in-opt" ]
    _errors_multiplyer = { "E": 75, "W": 95 }
    _log_file="rpmlint.log"

    def _check_licensecheck_output(self, output):
       """
       Processes pure output and returns score from 0 to 100
       """
       error_types_present = { "E": {}, "W": {} }
       
       # last line is resutl summary, ignore it
       for line in output.split("\n")[0:-1]:
           # some messages contain line number
           match = re.match(r"^(?P<pkgname>[^:]+):(\d+:)?\s+(?P<errtype>[WE]):"
                             "\s+(?P<errname>\S+)", line)
           if match:
               error_type = match.group("errtype")
               error_message = match.group("errname")
               if self._scl_name and  error_message in self._errors_to_ignore:
                   continue
               if error_message in self._errors_to_ignore:
                   continue
               if not error_message in error_types_present[error_type]:
                   error_types_present[error_type][error_message] = 1
               else:
                   error_types_present[error_type][error_message] += 1
               self._log_warning(line)

       score = 100
       for err in ["E", "W"]:
           for i in range(len(error_types_present[err].keys())):
               score = score * self._errors_multiplyer[err] / 100

       return score
           

    def perform(self):
        self._touch_log()
        for package in self._packages:
            sub_score = 0
            files_checked = 0
            try:
                # first extract RPM
                print("Calling %s %s" % ("rpmdev-extract", package))
                out = subprocess.check_output(["rpmdev-extract", package])

                # TODO: now un-wrap sources

                # then run licensecheck on whole directory
                extracted_dir = re.sub(r'(.*)+\.rpm', r'\1', os.path.basename(package))
                shutil.rmtree(extracted_dir)
                print("Calling %s %s %s" % ("licensecheck", "-r", extracted_dir))
                out = subprocess.check_output(["licensecheck", "-r", extracted_dir])
                    
            except subprocess.CalledProcessError as e:
                out = e.output
                
            files_checked += 1
            sub_score += self._check_licensecheck_output(out)

        if files_checked > 0:
            self._score = sub_score / files_checked
        else:
            self._score = 0

        return {"score": self._score}

